import React, {
  useEffect,
  useRef,
  useState,
  useCallback,
  ReactNode,
} from "react";
import { useNavigate } from "react-router-dom";
import { Map, View, Overlay, MapBrowserEvent } from "ol";
import TileLayer from "ol/layer/Tile";
import OSM from "ol/source/OSM";
import VectorLayer from "ol/layer/Vector";
import VectorSource, { VectorSourceEvent } from "ol/source/Vector";
import { Draw, Modify, Select } from "ol/interaction";
import { DrawEvent } from "ol/interaction/Draw";
import { ModifyEvent } from "ol/interaction/Modify";
import { SelectEvent } from "ol/interaction/Select";
import {
  Style,
  Fill,
  Stroke,
  Circle as CircleStyle,
  RegularShape,
} from "ol/style";
import { Point, LineString, Geometry } from "ol/geom";
import { transform } from "ol/proj";
import { GeoJSON } from "ol/format";
import {
  Bus,
  RouteIcon,
  MapPin,
  Trash2,
  ArrowRightLeft,
  Landmark,
  Pencil,
  Save,
  Plus,
  Loader2,
} from "lucide-react";
import { toast } from "react-toastify";
import { singleClick, doubleClick } from "ol/events/condition";
import { Feature } from "ol";
import Layer from "ol/layer/Layer";
import Source from "ol/source/Source";
import { FeatureLike } from "ol/Feature";

import mapStore from "./mapStore";
// --- API INTERFACES ---

// --- MOCK API ---

// --- CONFIGURATION ---
export const mapConfig = {
  center: [30.311, 59.94] as [number, number],
  zoom: 13,
};

// --- SVG ICONS ---
const EditIcon = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-4 w-4 mr-1 sm:mr-2"
    fill="none"
    viewBox="0 0 24 24"
    stroke="currentColor"
  >
    <path
      strokeLinecap="round"
      strokeLinejoin="round"
      strokeWidth={2}
      d="M15.232 5.232l3.536 3.536m-2.036-5.036a2.5 2.5 0 113.536 3.536L6.5 21.036H3v-3.572L16.732 3.732z"
    />
  </svg>
);
const LineIconSvg = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-4 w-4 mr-1 sm:mr-2"
    fill="none"
    viewBox="0 0 24 24"
    stroke="currentColor"
  >
    <path
      strokeLinecap="round"
      strokeLinejoin="round"
      strokeWidth={2}
      d="M3.055 11H5a2 2 0 012 2v1a2 2 0 002 2h10a2 2 0 002-2v-1a2 2 0 012-2h1.945M7.732 4.064A2.5 2.5 0 105.23 6.24m13.54 0a2.5 2.5 0 10-2.502-2.176M12 16.05V21m0-17.948V3"
    />
  </svg>
);

// --- TYPE DEFINITIONS ---
interface MapServiceConfig {
  target: HTMLElement;
  center: [number, number];
  zoom: number;
}

interface HistoryState {
  action: string;
  state: string;
}

type FeatureType = "station" | "route" | "sight";

class MapService {
  private map: Map | null;
  private vectorSource: VectorSource<Feature<Geometry>>;
  private vectorLayer: VectorLayer<VectorSource<Feature<Geometry>>>;
  private tooltipElement: HTMLElement;
  private tooltipOverlay: Overlay | null;
  private mode: string | null;
  private currentDrawingType: "Point" | "LineString" | null;
  private currentDrawingFeatureType: FeatureType | null;
  private currentInteraction: Draw | null;
  private modifyInteraction: Modify;
  private selectInteraction: Select;
  private hoveredFeatureId: string | number | null;
  private history: HistoryState[];
  private historyIndex: number;
  private beforeModifyState: string | null;
  private boundHandlePointerMove: (
    event: MapBrowserEvent<PointerEvent>
  ) => void;
  private boundHandlePointerLeave: () => void;
  private boundHandleContextMenu: (event: MouseEvent) => void;
  private boundHandleKeyDown: (event: KeyboardEvent) => void;

  // Styles
  private defaultStyle: Style;
  private selectedStyle: Style;
  private drawStyle: Style;
  private busIconStyle: Style;
  private selectedBusIconStyle: Style;
  private drawBusIconStyle: Style;
  private sightIconStyle: Style;
  private selectedSightIconStyle: Style;
  private drawSightIconStyle: Style;
  private universalHoverStylePoint: Style;
  private hoverSightIconStyle: Style;
  private universalHoverStyleLine: Style;

  // Callbacks
  private setLoading: (loading: boolean) => void;
  private setError: (error: string | null) => void;
  private onModeChangeCallback: (mode: string) => void;
  private onFeaturesChange: (features: Feature<Geometry>[]) => void;
  private onFeatureSelect: (feature: Feature<Geometry> | null) => void;

  constructor(
    config: MapServiceConfig,
    setLoading: (loading: boolean) => void,
    setError: (error: string | null) => void,
    onModeChangeCallback: (mode: string) => void,
    onFeaturesChange: (features: Feature<Geometry>[]) => void,
    onFeatureSelect: (feature: Feature<Geometry> | null) => void,
    tooltipElement: HTMLElement
  ) {
    this.map = null;
    this.tooltipElement = tooltipElement;
    this.tooltipOverlay = null;
    this.mode = null;
    this.currentDrawingType = null;
    this.currentDrawingFeatureType = null;
    this.currentInteraction = null;
    this.hoveredFeatureId = null;
    this.history = [];
    this.historyIndex = -1;
    this.beforeModifyState = null;

    this.setLoading = setLoading;
    this.setError = setError;
    this.onModeChangeCallback = onModeChangeCallback;
    this.onFeaturesChange = onFeaturesChange;
    this.onFeatureSelect = onFeatureSelect;

    this.defaultStyle = new Style({
      fill: new Fill({ color: "rgba(66, 153, 225, 0.2)" }),
      stroke: new Stroke({ color: "#3182ce", width: 3 }),
    });
    this.selectedStyle = new Style({
      fill: new Fill({ color: "rgba(221, 107, 32, 0.3)" }),
      stroke: new Stroke({ color: "#dd6b20", width: 4 }),
      image: new CircleStyle({
        radius: 6,
        fill: new Fill({ color: "#dd6b20" }),
        stroke: new Stroke({ color: "white", width: 1.5 }),
      }),
    });
    this.drawStyle = new Style({
      fill: new Fill({ color: "rgba(74, 222, 128, 0.3)" }),
      stroke: new Stroke({
        color: "rgba(34, 197, 94, 0.7)",
        width: 2,
        lineDash: [5, 5],
      }),
      image: new CircleStyle({
        radius: 5,
        fill: new Fill({ color: "rgba(34, 197, 94, 0.7)" }),
      }),
    });

    this.busIconStyle = new Style({
      image: new CircleStyle({
        radius: 8,
        fill: new Fill({ color: "rgba(0, 60, 255, 0.8)" }),
        stroke: new Stroke({ color: "#ffffff", width: 1.5 }),
      }),
    });
    this.selectedBusIconStyle = new Style({
      image: new CircleStyle({
        radius: 10,
        fill: new Fill({ color: "rgba(221, 107, 32, 0.9)" }),
        stroke: new Stroke({ color: "#ffffff", width: 2 }),
      }),
    });
    this.drawBusIconStyle = new Style({
      image: new CircleStyle({
        radius: 8,
        fill: new Fill({ color: "rgba(100, 180, 100, 0.8)" }),
        stroke: new Stroke({ color: "#ffffff", width: 1.5 }),
      }),
    });

    this.sightIconStyle = new Style({
      image: new RegularShape({
        fill: new Fill({ color: "rgba(139, 92, 246, 0.8)" }),
        stroke: new Stroke({ color: "#ffffff", width: 2 }),
        points: 5,
        radius: 12,
        radius2: 6,
        angle: 0,
      }),
    });
    this.selectedSightIconStyle = new Style({
      image: new RegularShape({
        fill: new Fill({ color: "rgba(221, 107, 32, 0.9)" }),
        stroke: new Stroke({ color: "#ffffff", width: 2.5 }),
        points: 5,
        radius: 14,
        radius2: 7,
        angle: 0,
      }),
    });
    this.drawSightIconStyle = new Style({
      image: new RegularShape({
        fill: new Fill({ color: "rgba(100, 180, 100, 0.8)" }),
        stroke: new Stroke({ color: "#ffffff", width: 1.5 }),
        points: 5,
        radius: 12,
        radius2: 6,
        angle: 0,
      }),
    });

    this.universalHoverStylePoint = new Style({
      image: new CircleStyle({
        radius: 11,
        fill: new Fill({ color: "rgba(255, 165, 0, 0.7)" }),
        stroke: new Stroke({ color: "rgba(255,255,255,0.8)", width: 2 }),
      }),
      zIndex: Infinity,
    });
    this.hoverSightIconStyle = new Style({
      image: new RegularShape({
        fill: new Fill({ color: "rgba(255, 165, 0, 0.7)" }),
        stroke: new Stroke({ color: "rgba(255,255,255,0.8)", width: 2 }),
        points: 5,
        radius: 15,
        radius2: 7.5,
        angle: 0,
      }),
      zIndex: Infinity,
    });
    this.universalHoverStyleLine = new Style({
      stroke: new Stroke({ color: "rgba(255, 165, 0, 0.8)", width: 5 }),
      zIndex: Infinity,
    });

    this.vectorSource = new VectorSource<Feature<Geometry>>();
    this.vectorLayer = new VectorLayer({
      source: this.vectorSource,
      style: (featureLike: FeatureLike) => {
        const feature = featureLike as Feature<Geometry>;
        if (!feature) return this.defaultStyle;

        const geometryType = feature.getGeometry()?.getType();
        const fId = feature.getId();
        const featureType = feature.get("featureType");

        const isEditSelected = this.selectInteraction
          ?.getFeatures()
          .getArray()
          .includes(feature);
        const isHovered = this.hoveredFeatureId === fId;

        if (geometryType === "Point") {
          const defaultPointStyle =
            featureType === "sight" ? this.sightIconStyle : this.busIconStyle;
          const selectedPointStyle =
            featureType === "sight"
              ? this.selectedSightIconStyle
              : this.selectedBusIconStyle;

          if (isEditSelected) {
            return selectedPointStyle;
          }
          if (isHovered) {
            return featureType === "sight"
              ? this.hoverSightIconStyle
              : this.universalHoverStylePoint;
          }
          return defaultPointStyle;
        } else if (geometryType === "LineString") {
          if (isEditSelected) {
            return this.selectedStyle;
          }
          if (isHovered) {
            return this.universalHoverStyleLine;
          }
          return this.defaultStyle;
        }

        return this.defaultStyle;
      },
    });

    this.boundHandlePointerMove = this.handlePointerMove.bind(this);
    this.boundHandlePointerLeave = this.handlePointerLeave.bind(this);
    this.boundHandleContextMenu = this.handleContextMenu.bind(this);
    this.boundHandleKeyDown = this.handleKeyDown.bind(this);

    this.vectorSource.on(
      "addfeature",
      this.handleFeatureEvent.bind(this) as any
    );
    this.vectorSource.on("removefeature", () => this.updateFeaturesInReact());
    this.vectorSource.on(
      "changefeature",
      this.handleFeatureChange.bind(this) as any
    );

    let renderCompleteHandled = false;
    const MAP_LOAD_TIMEOUT = 15000;
    try {
      this.map = new Map({
        target: config.target,
        layers: [new TileLayer({ source: new OSM() }), this.vectorLayer],
        view: new View({
          center: transform(config.center, "EPSG:4326", "EPSG:3857"),
          zoom: config.zoom,
        }),
        controls: [],
      });
      if (this.tooltipElement && this.map) {
        this.tooltipOverlay = new Overlay({
          element: this.tooltipElement,
          offset: [15, 0],
          positioning: "center-left",
        });
        this.map.addOverlay(this.tooltipOverlay);
      }
      this.map.once("rendercomplete", () => {
        if (!renderCompleteHandled) {
          this.setLoading(false);
          renderCompleteHandled = true;
          setError(null);
        }
      });
      setTimeout(() => {
        if (!renderCompleteHandled && this.map) {
          this.setLoading(false);
          this.setError("Карта не загрузилась вовремя.");
          renderCompleteHandled = true;
        }
      }, MAP_LOAD_TIMEOUT);
    } catch (error) {
      this.setError("Критическая ошибка при инициализации карты.");
      this.setLoading(false);
      renderCompleteHandled = true;
    }

    this.modifyInteraction = new Modify({
      source: this.vectorSource,
      style: (feature) => {
        const originalFeature = feature.get("features")[0];
        if (
          originalFeature &&
          originalFeature.getGeometry()?.getType() === "Point"
        ) {
          return null;
        }
        return new Style({
          image: new CircleStyle({
            radius: 5,
            fill: new Fill({
              color: "rgba(255, 255, 255, 0.7)",
            }),
            stroke: new Stroke({
              color: "#0099ff",
              width: 2,
            }),
          }),
        });
      },
      deleteCondition: (e: MapBrowserEvent<any>) => doubleClick(e),
    });

    this.selectInteraction = new Select({
      style: (featureLike: FeatureLike) => {
        if (!featureLike || !featureLike.getGeometry) return this.defaultStyle;
        const feature = featureLike as Feature<Geometry>;
        const featureType = feature.get("featureType");
        const geometryType = feature.getGeometry()?.getType();

        if (geometryType === "Point") {
          return featureType === "sight"
            ? this.selectedSightIconStyle
            : this.selectedBusIconStyle;
        }
        return this.selectedStyle;
      },
      condition: (e: MapBrowserEvent<any>) => {
        const isEdit = this.mode === "edit";
        const isSingle = singleClick(e);
        if (!isEdit || !isSingle) return false;

        let clickModify = false;
        if (this.modifyInteraction.getActive() && this.map) {
          const px = e.pixel;
          const internalModify = this.modifyInteraction as Modify;
          const sketchFs: Feature<Point>[] | undefined = (
            internalModify as any
          ).overlay_
            ?.getSource()
            ?.getFeatures();

          if (sketchFs) {
            for (const sf of sketchFs) {
              const g = sf.getGeometry();
              if (g) {
                const coord = this.map.getCoordinateFromPixel(px);
                if (!coord) continue;
                const cp = g.getClosestPoint(coord);
                const cppx = this.map.getPixelFromCoordinate(cp);
                if (!cppx) continue;
                const pixelTolerance =
                  (internalModify as any).pixelTolerance_ || 10;
                if (
                  Math.sqrt((px[0] - cppx[0]) ** 2 + (px[1] - cppx[1]) ** 2) <
                  pixelTolerance + 2
                ) {
                  clickModify = true;
                  break;
                }
              }
            }
          }
        }
        return !clickModify;
      },
      filter: (_: FeatureLike, l: Layer<Source, any> | null) =>
        l === this.vectorLayer,
    });

    this.modifyInteraction.on("modifystart", () => {
      const geoJSONFormat = new GeoJSON();
      if (!this.map) return;
      this.beforeModifyState = geoJSONFormat.writeFeatures(
        this.vectorSource.getFeatures(),
        {
          dataProjection: "EPSG:4326",
          featureProjection: this.map.getView().getProjection().getCode(),
        }
      );
    });

    this.modifyInteraction.on("modifyend", (event: ModifyEvent) => {
      if (this.beforeModifyState) {
        this.addStateToHistory("modify-before", this.beforeModifyState);
        this.beforeModifyState = null;
      }
      this.updateFeaturesInReact();
    });

    if (this.map) {
      this.map.addInteraction(this.modifyInteraction);
      this.map.addInteraction(this.selectInteraction);
      this.modifyInteraction.setActive(false);
      this.selectInteraction.setActive(false);

      this.selectInteraction.on("select", (e: SelectEvent) => {
        if (this.mode === "edit") {
          const selFs = e.selected as Feature<Geometry>[];
          this.modifyInteraction.setActive(selFs.length > 0);
          if (selFs.length > 0) {
            this.onFeatureSelect(selFs[0]);
          } else {
            this.onFeatureSelect(null);
          }
        }
      });
      this.map.on("pointermove", this.boundHandlePointerMove as any);
      const targetEl = this.map.getTargetElement();
      if (targetEl instanceof HTMLElement) {
        targetEl.addEventListener("contextmenu", this.boundHandleContextMenu);
        targetEl.addEventListener("pointerleave", this.boundHandlePointerLeave);
      }
      document.addEventListener("keydown", this.boundHandleKeyDown);
      this.activateEditMode();
    }
  }

  public saveMapState(): void {
    const geoJSON = this.getAllFeaturesAsGeoJSON();
    if (geoJSON) {
      console.log("Сохранение состояния карты (GeoJSON):", geoJSON);
      alert(
        "Данные карты выведены в консоль разработчика и готовы к отправке!"
      );
    } else {
      alert("Нет объектов для сохранения.");
    }
  }

  public loadFeaturesFromApi(
    apiStations: typeof mapStore.stations,
    apiRoutes: typeof mapStore.routes,
    apiSights: typeof mapStore.sights
  ): void {
    if (!this.map) return;

    const projection = this.map.getView().getProjection();
    const featuresToAdd: Feature<Geometry>[] = [];

    apiStations.forEach((station) => {
      if (station.longitude == null || station.latitude == null) {
        console.warn(`Station ${station.id} has null coordinates, skipping...`);
        return;
      }
      const point = new Point(
        transform(
          [station.longitude, station.latitude],
          "EPSG:4326",
          projection
        )
      );
      const feature = new Feature({ geometry: point, name: station.name });
      feature.setId(`station-${station.id}`);
      feature.set("featureType", "station");
      featuresToAdd.push(feature);
    });

    apiRoutes.forEach((route) => {
      if (!route.path || route.path.length === 0) {
        console.warn(`Route ${route.id} has no path coordinates, skipping...`);
        return;
      }
      const coordinates = route.path
        .filter((coord) => coord[0] != null && coord[1] != null)
        .map((coord) => transform(coord, "EPSG:4326", projection));
      if (coordinates.length === 0) {
        console.warn(
          `Route ${route.id} has no valid coordinates after filtering, skipping...`
        );
        return;
      }
      const line = new LineString(coordinates);
      const feature = new Feature({ geometry: line, name: route.route_number });
      feature.setId(`route-${route.id}`);
      feature.set("featureType", "route");
      featuresToAdd.push(feature);
    });

    apiSights.forEach((sight) => {
      if (sight.longitude == null || sight.latitude == null) {
        console.warn(`Sight ${sight.id} has null coordinates, skipping...`);
        return;
      }
      const point = new Point(
        transform([sight.longitude, sight.latitude], "EPSG:4326", projection)
      );
      const feature = new Feature({
        geometry: point,
        name: sight.name,
        description: sight.description,
      });
      feature.setId(`sight-${sight.id}`);
      feature.set("featureType", "sight");
      featuresToAdd.push(feature);
    });

    this.vectorSource.addFeatures(featuresToAdd);
    this.updateFeaturesInReact();
  }

  private addStateToHistory(
    actionDescription: string,
    stateToSave: string
  ): void {
    this.history = this.history.slice(0, this.historyIndex + 1);
    this.history.push({ action: actionDescription, state: stateToSave });
    this.historyIndex = this.history.length - 1;
  }

  private getCurrentStateAsGeoJSON(): string | null {
    if (!this.map) return null;
    const geoJSONFormat = new GeoJSON();
    return geoJSONFormat.writeFeatures(this.vectorSource.getFeatures(), {
      dataProjection: "EPSG:4326",
      featureProjection: this.map.getView().getProjection().getCode(),
    });
  }

  public undo(): void {
    if (this.historyIndex >= 0) {
      const stateToRestore = this.history[this.historyIndex].state;
      this.applyHistoryState(stateToRestore);
      this.historyIndex--;
    } else {
      this.vectorSource.clear();
      this.updateFeaturesInReact();
      this.onFeatureSelect(null);
    }
  }

  public redo(): void {
    if (this.historyIndex < this.history.length - 1) {
      this.historyIndex++;
      this.applyHistoryState(this.history[this.historyIndex].state);
    }
  }

  private applyHistoryState(geoJSONState: string): void {
    if (!this.map) return;
    const geoJSONFormat = new GeoJSON();
    const features = geoJSONFormat.readFeatures(geoJSONState, {
      dataProjection: "EPSG:4326",
      featureProjection: this.map.getView().getProjection().getCode(),
    }) as Feature<Geometry>[];
    this.vectorSource.clear();
    if (features.length > 0) this.vectorSource.addFeatures(features);
    this.updateFeaturesInReact();
    this.onFeatureSelect(null);
    this.selectInteraction.getFeatures().clear();
    this.vectorLayer.changed();
  }

  private updateFeaturesInReact(): void {
    if (this.onFeaturesChange) {
      this.onFeaturesChange(this.vectorSource.getFeatures());
    }
  }

  private handlePointerLeave(): void {
    if (this.hoveredFeatureId) {
      this.hoveredFeatureId = null;
      this.vectorLayer.changed();
    }
    if (this.tooltipOverlay) this.tooltipOverlay.setPosition(undefined);
  }

  private handleKeyDown(event: KeyboardEvent): void {
    if ((event.ctrlKey || event.metaKey) && event.key === "z") {
      event.preventDefault();
      this.undo();
      return;
    }
    if ((event.ctrlKey || event.metaKey) && event.key === "y") {
      event.preventDefault();
      this.redo();
      return;
    }
    if (event.key === "Escape") {
      if (this.mode && this.mode.startsWith("drawing-")) this.finishDrawing();
      else if (
        this.mode === "edit" &&
        this.selectInteraction?.getFeatures().getLength() > 0
      ) {
        this.selectInteraction.getFeatures().clear();
        this.onFeatureSelect(null);
      }
    }
  }

  private setMode(newMode: string): void {
    if (!this.map) return;
    const oldMode = this.mode;
    this.mode = newMode;

    if (this.hoveredFeatureId && oldMode !== newMode) {
      this.hoveredFeatureId = null;
      this.vectorLayer.changed();
      if (this.tooltipOverlay) this.tooltipOverlay.setPosition(undefined);
    }

    if (this.onModeChangeCallback) this.onModeChangeCallback(newMode);

    if (this.currentInteraction instanceof Draw) {
      this.map.removeInteraction(this.currentInteraction);
      this.currentInteraction = null;
    }

    if (newMode === "edit") {
      this.selectInteraction.setActive(true);
    } else {
      this.selectInteraction.getFeatures().clear();
      this.selectInteraction.setActive(false);
      this.modifyInteraction.setActive(false);
    }
  }

  public activateEditMode(): void {
    this.currentDrawingType = null;
    this.setMode("edit");
    if (this.selectInteraction.getFeatures().getLength() > 0) {
      const firstSelectedFeature = this.selectInteraction
        .getFeatures()
        .item(0) as Feature<Geometry>;
      this.onFeatureSelect(firstSelectedFeature);
    } else {
      this.onFeatureSelect(null);
    }
  }

  public startDrawing(
    type: "Point" | "LineString",
    featureType: FeatureType
  ): void {
    if (!this.map) return;

    this.currentDrawingType = type;
    this.currentDrawingFeatureType = featureType;

    const drawingMode = `drawing-${featureType}`;
    this.setMode(drawingMode);

    if (this.currentInteraction instanceof Draw) {
      this.map.removeInteraction(this.currentInteraction);
    }

    let styleForDrawing: Style;
    if (featureType === "station") {
      styleForDrawing = this.drawBusIconStyle;
    } else if (featureType === "sight") {
      styleForDrawing = this.drawSightIconStyle;
    } else {
      styleForDrawing = this.drawStyle;
    }

    this.currentInteraction = new Draw({
      source: this.vectorSource,
      type,
      style: styleForDrawing,
    });

    let stateBeforeDraw: string | null = null;
    this.currentInteraction.on("drawstart", () => {
      stateBeforeDraw = this.getCurrentStateAsGeoJSON();
    });

    this.currentInteraction.on("drawend", (event: DrawEvent) => {
      if (stateBeforeDraw) {
        this.addStateToHistory("draw-before", stateBeforeDraw);
      }
      const feature = event.feature as Feature<Geometry>;
      const fType = this.currentDrawingFeatureType;
      if (!fType) return;

      feature.set("featureType", fType);

      let baseName = "";
      let namePrefix = "";

      if (fType === "station") {
        baseName = "Станция";
        namePrefix = "Станция ";
      } else if (fType === "sight") {
        baseName = "Достопримечательность";
        namePrefix = "Достопримечательность ";
      } else if (fType === "route") {
        baseName = "Маршрут";
        namePrefix = "Маршрут ";
      }

      const existingNamedFeatures = this.vectorSource
        .getFeatures()
        .filter(
          (f) =>
            f !== feature &&
            f.get("featureType") === fType &&
            f.get("name") &&
            (f.get("name") as string).startsWith(namePrefix)
        );

      let maxNumber = 0;
      existingNamedFeatures.forEach((f) => {
        const name = f.get("name") as string;
        if (name) {
          const numStr = name.substring(namePrefix.length);
          const num = parseInt(numStr, 10);
          if (!isNaN(num) && num > maxNumber) maxNumber = num;
        }
      });

      const newNumber = maxNumber + 1;
      feature.set("name", `${baseName} ${newNumber}`);

      if (this.currentDrawingType === "LineString") {
        this.finishDrawing();
      }
    });

    this.map.addInteraction(this.currentInteraction);
  }

  public startDrawingMarker(): void {
    this.startDrawing("Point", "station");
  }
  public startDrawingLine(): void {
    this.startDrawing("LineString", "route");
  }
  public startDrawingSight(): void {
    this.startDrawing("Point", "sight");
  }

  public finishDrawing(): void {
    if (!this.map || !this.currentInteraction) return;
    const drawInteraction = this.currentInteraction as Draw;
    if ((drawInteraction as any).sketchFeature_) {
      try {
        this.currentInteraction.finishDrawing();
      } catch (e) {
        // Drawing could not be finished
      }
    }
    this.map.removeInteraction(this.currentInteraction);
    this.currentInteraction = null;
    this.currentDrawingType = null;
    this.currentDrawingFeatureType = null;
    this.activateEditMode();
  }

  private handleContextMenu(event: MouseEvent): void {
    event.preventDefault();
    if (
      !this.map ||
      !(
        this.mode &&
        this.mode.startsWith("drawing-") &&
        this.currentInteraction instanceof Draw
      )
    ) {
      return;
    }
    const drawInteraction = this.currentInteraction as Draw;
    if (
      this.currentDrawingType === "LineString" &&
      (drawInteraction as any).sketchFeature_
    ) {
      try {
        this.currentInteraction.finishDrawing();
      } catch (e) {
        this.finishDrawing();
      }
    } else {
      this.finishDrawing();
    }
  }

  private handlePointerMove(event: MapBrowserEvent<PointerEvent>): void {
    if (!this.map || event.dragging) {
      if (this.hoveredFeatureId) {
        this.hoveredFeatureId = null;
        this.vectorLayer.changed();
      }
      if (this.tooltipOverlay) this.tooltipOverlay.setPosition(undefined);
      return;
    }

    const pixel = this.map.getEventPixel(event.originalEvent as PointerEvent);
    const featureAtPixel: Feature<Geometry> | undefined =
      this.map.forEachFeatureAtPixel(
        pixel,
        (f: FeatureLike) => f as Feature<Geometry>,
        { layerFilter: (l) => l === this.vectorLayer, hitTolerance: 5 }
      );

    const newHoveredFeatureId = featureAtPixel ? featureAtPixel.getId() : null;

    if (this.tooltipOverlay && this.tooltipElement) {
      if (this.mode === "edit" && featureAtPixel) {
        const name = featureAtPixel.get("name");
        if (name) {
          this.tooltipElement.innerHTML = name as string;
          this.tooltipOverlay.setPosition(event.coordinate);
        } else {
          this.tooltipOverlay.setPosition(undefined);
        }
      } else {
        this.tooltipOverlay.setPosition(undefined);
      }
    }

    if (this.hoveredFeatureId !== newHoveredFeatureId) {
      this.hoveredFeatureId = newHoveredFeatureId as string | number | null;
      this.vectorLayer.changed();
    }
  }

  public selectFeature(featureId: string | number | undefined): void {
    if (!this.map || featureId === undefined) {
      this.onFeatureSelect(null);
      if (this.mode === "edit") this.selectInteraction.getFeatures().clear();
      this.vectorLayer.changed();
      return;
    }

    const feature = this.vectorSource.getFeatureById(featureId);
    if (!feature) {
      this.onFeatureSelect(null);
      if (this.mode === "edit") this.selectInteraction.getFeatures().clear();
      this.vectorLayer.changed();
      return;
    }

    if (this.mode === "edit") {
      this.selectInteraction.getFeatures().clear();
      this.selectInteraction.getFeatures().push(feature);
      this.onFeatureSelect(feature);
    }
    this.vectorLayer.changed();

    const view = this.map.getView();
    const geometry = feature.getGeometry();
    if (geometry) {
      if (geometry instanceof Point) {
        view.animate({
          center: geometry.getCoordinates(),
          duration: 500,
          zoom: Math.max(view.getZoom() || mapConfig.zoom, 14),
        });
      } else {
        view.fit(geometry.getExtent(), {
          duration: 500,
          padding: [50, 50, 50, 50],
          maxZoom: 15,
        });
      }
    }
  }

  public deleteFeature(
    featureId: string | number | undefined,
    recourse: string
  ): void {
    if (featureId === undefined) return;
    const id = featureId.toString().split("-")[1];
    if (recourse) {
      mapStore.deleteRecourse(recourse, Number(id));
      toast.success("Объект успешно удален");
    }
    const feature = this.vectorSource.getFeatureById(featureId);
    if (feature) {
      const currentState = this.getCurrentStateAsGeoJSON();
      if (currentState) this.addStateToHistory("delete", currentState);

      const selectedFeaturesCollection = this.selectInteraction?.getFeatures();
      if (selectedFeaturesCollection?.getArray().includes(feature)) {
        selectedFeaturesCollection.clear();
        this.onFeatureSelect(null);
      }

      this.vectorSource.removeFeature(feature);
      this.vectorLayer.changed();
    }
  }

  // --- ИСПРАВЛЕННЫЙ МЕТОД ---
  public getAllFeaturesAsGeoJSON(): string | null {
    if (!this.vectorSource || !this.map) return null;
    const feats = this.vectorSource.getFeatures();
    if (feats.length === 0) return null;

    const geoJSONFmt = new GeoJSON();

    // Просто передаем опции трансформации в метод writeFeatures.
    // Он сам всё сделает правильно, не трогая оригинальные объекты.
    return geoJSONFmt.writeFeatures(feats, {
      dataProjection: "EPSG:4326", // В какую проекцию конвертировать (стандарт для GeoJSON)
      featureProjection: this.map.getView().getProjection(), // В какой проекции находятся объекты на карте
    });
  }

  public destroy(): void {
    if (this.map) {
      document.removeEventListener("keydown", this.boundHandleKeyDown);
      const targetEl = this.map.getTargetElement();
      if (targetEl instanceof HTMLElement) {
        targetEl.removeEventListener(
          "contextmenu",
          this.boundHandleContextMenu
        );
        targetEl.removeEventListener(
          "pointerleave",
          this.boundHandlePointerLeave
        );
      }
      this.map.un("pointermove", this.boundHandlePointerMove as any);
      if (this.tooltipOverlay) this.map.removeOverlay(this.tooltipOverlay);
      this.vectorSource.clear();
      this.map.setTarget(undefined);
      this.map = null;
    }
  }

  private handleFeatureEvent(
    event: VectorSourceEvent<Feature<Geometry>>
  ): void {
    if (!event.feature) return;
    const feature = event.feature;
    if (!feature.getId()) {
      feature.setId(Date.now() + Math.random().toString(36).substr(2, 9));
    }
    this.updateFeaturesInReact();
  }

  private handleFeatureChange(
    event: VectorSourceEvent<Feature<Geometry>>
  ): void {
    if (!event.feature) return;
    this.updateFeaturesInReact();
  }
}

// --- MAP CONTROLS COMPONENT ---
interface MapControlsProps {
  mapService: MapService | null;
  activeMode: string;
}

const MapControls: React.FC<MapControlsProps> = ({
  mapService,
  activeMode,
}) => {
  if (!mapService) return null;

  const controls = [
    {
      mode: "edit",
      title: "Редактировать",
      longTitle: "Редактирование",
      icon: <EditIcon />,
      action: () => mapService.activateEditMode(),
    },
    {
      mode: "drawing-station",
      title: "Станция",
      longTitle: "Добавить станцию",
      icon: <Bus size={16} className="mr-1 sm:mr-2" />,
      action: () => mapService.startDrawingMarker(),
    },
    {
      mode: "drawing-sight",
      title: "Достопримечательность",
      longTitle: "Добавить достопримечательность",
      icon: <Landmark size={16} className="mr-1 sm:mr-2" />,
      action: () => mapService.startDrawingSight(),
    },
    {
      mode: "drawing-route",
      title: "Маршрут",
      longTitle: "Добавить маршрут",
      icon: <LineIconSvg />,
      action: () => mapService.startDrawingLine(),
    },
  ];
  return (
    <div className="absolute top-4 left-1/2 -translate-x-1/2 z-20 flex flex-nowrap justify-center p-2 bg-white/90 backdrop-blur-sm rounded-lg shadow-xl space-x-1 sm:space-x-2">
      {controls.map((c) => (
        <button
          key={c.mode}
          className={`flex items-center px-3 py-2 rounded-md transition-all duration-200 text-xs sm:text-sm font-medium focus:outline-none focus:ring-2 focus:ring-offset-1 focus:ring-blue-500 focus:ring-opacity-75 ${
            activeMode === c.mode
              ? "bg-blue-600 text-white shadow-md hover:bg-blue-700"
              : "bg-gray-100 text-gray-700 hover:bg-blue-100 hover:text-blue-700"
          }`}
          onClick={c.action}
          title={c.longTitle}
        >
          {c.icon}
          <span className="hidden sm:inline ml-0 sm:ml-1">{c.title}</span>
        </button>
      ))}
    </div>
  );
};

// --- MAP SIGHTBAR COMPONENT ---
interface MapSightbarProps {
  mapService: MapService | null;
  mapFeatures: Feature<Geometry>[];
  selectedFeature: Feature<Geometry> | null;
}

const MapSightbar: React.FC<MapSightbarProps> = ({
  mapService,
  mapFeatures,
  selectedFeature,
}) => {
  const [activeSection, setActiveSection] = useState<string | null>("layers");
  const navigate = useNavigate();
  const [isLoading, setIsLoading] = useState(false);

  const toggleSection = (id: string) =>
    setActiveSection(activeSection === id ? null : id);

  const handleFeatureClick = useCallback(
    (id: string | number | undefined) => {
      if (mapService) mapService.selectFeature(id);
    },
    [mapService]
  );

  const handleDeleteFeature = useCallback(
    (id: string | number | undefined, recourse: string) => {
      if (
        mapService &&
        window.confirm(
          "Вы действительно хотите удалить этот объект? Утерянные данные не могут быть восстановлены."
        )
      )
        mapService.deleteFeature(id, recourse);
    },
    [mapService]
  );

  const handleEditFeature = useCallback(
    (featureType: string | undefined, fullId: string | number | undefined) => {
      if (!featureType || !fullId) return;
      const numericId = String(fullId).split("-")[1];
      if (!numericId) {
        console.error("Could not parse numeric ID from", fullId);
        return;
      }
      navigate(`/${featureType}/${numericId}/edit`);
    },
    [navigate]
  );

  const handleSave = useCallback(async () => {
    const geoJSON = mapService?.getAllFeaturesAsGeoJSON();
    if (geoJSON) {
      setIsLoading(true);
      await mapStore.handleSave(geoJSON);
      toast.success("Данные успешно сохранены");
    } else {
      toast.error("Ошибка при сохранении данных");
    }
    setIsLoading(false);
  }, [mapService]);

  const stations = mapFeatures.filter(
    (f) =>
      f.get("featureType") === "station" ||
      (f.getGeometry()?.getType() === "Point" && !f.get("featureType"))
  );
  const lines = mapFeatures.filter(
    (f) =>
      f.get("featureType") === "route" ||
      (f.getGeometry()?.getType() === "LineString" && !f.get("featureType"))
  );
  const sights = mapFeatures.filter((f) => f.get("featureType") === "sight");

  interface SidebarSection {
    id: string;
    title: string;
    icon: ReactNode;
    content: ReactNode;
  }

  const sections: SidebarSection[] = [
    {
      id: "layers",
      title: `Остановки (${stations.length})`,
      icon: <Bus size={20} />,
      content: (
        <div className="space-y-1 max-h-60 overflow-y-auto pr-1">
          {stations.length > 0 ? (
            stations.map((s) => {
              const sId = s.getId();
              const sName = (s.get("name") as string) || "Без названия";
              const isSelected = selectedFeature?.getId() === sId;

              return (
                <div
                  key={String(sId)}
                  className={`flex items-start justify-between p-2 rounded-md cursor-pointer group transition-colors duration-150 ${
                    isSelected
                      ? "bg-orange-100 border border-orange-300 hover:bg-orange-200"
                      : "hover:bg-blue-50"
                  }`}
                  onClick={() => handleFeatureClick(sId)}
                >
                  <div className="flex flex-col text-gray-800 text-sm flex-grow mr-2 min-w-0">
                    <div className="flex items-center">
                      <MapPin
                        size={16}
                        className={`mr-1.5 flex-shrink-0 ${
                          isSelected
                            ? "text-orange-500"
                            : "text-blue-500 group-hover:text-blue-600"
                        }`}
                      />
                      <span
                        className={`font-medium truncate ${
                          isSelected
                            ? "text-orange-600"
                            : "group-hover:text-blue-600"
                        }`}
                        title={sName}
                      >
                        {sName}
                      </span>
                    </div>
                  </div>
                  <div className="flex-shrink-0 flex items-center space-x-1 opacity-60 group-hover:opacity-100">
                    <button
                      onClick={(e) => {
                        e.stopPropagation();
                        handleEditFeature(s.get("featureType"), sId);
                      }}
                      className="p-1 rounded-full text-gray-400 hover:text-blue-600 hover:bg-blue-100 transition-colors"
                      title="Редактировать детали"
                    >
                      <Pencil size={16} />
                    </button>
                    <button
                      onClick={(e) => {
                        e.stopPropagation();
                        handleDeleteFeature(sId, "station");
                      }}
                      className="p-1 rounded-full text-gray-400 hover:text-red-600 hover:bg-red-100 transition-colors"
                      title="Удалить остановку"
                    >
                      <Trash2 size={16} />
                    </button>
                  </div>
                </div>
              );
            })
          ) : (
            <p className="text-gray-500 text-sm">Нет добавленных остановок.</p>
          )}
        </div>
      ),
    },
    {
      id: "lines",
      title: `Маршруты (${lines.length})`,
      icon: <RouteIcon size={20} />,
      content: (
        <div className="space-y-1 max-h-60 overflow-y-auto pr-1">
          {lines.length > 0 ? (
            lines.map((l) => {
              const lId = l.getId();
              const lName = (l.get("name") as string) || "Без названия";
              const isSelected = selectedFeature?.getId() === lId;
              const lGeom = l.getGeometry();
              let lineLengthText: string | null = null;
              if (lGeom instanceof LineString) {
                const length = lGeom.getLength();
                lineLengthText = `Длина: ${length.toFixed(1)} м`;
              }

              return (
                <div
                  key={String(lId)}
                  className={`flex items-start justify-between p-2 rounded-md cursor-pointer group transition-colors duration-150 ${
                    isSelected
                      ? "bg-orange-100 border border-orange-300 hover:bg-orange-200"
                      : "hover:bg-blue-50"
                  }`}
                  onClick={() => handleFeatureClick(lId)}
                >
                  <div className="flex flex-col text-gray-800 text-sm flex-grow mr-2 min-w-0">
                    <div className="flex items-center mb-0.5">
                      <ArrowRightLeft
                        size={16}
                        className={`mr-1.5 flex-shrink-0 ${
                          isSelected
                            ? "text-orange-500"
                            : "text-purple-500 group-hover:text-purple-600"
                        }`}
                      />
                      <span
                        className={`font-medium truncate ${
                          isSelected
                            ? "text-orange-600"
                            : "group-hover:text-purple-600"
                        }`}
                        title={lName}
                      >
                        {lName}
                      </span>
                    </div>
                    {lineLengthText && (
                      <p className="text-xs text-gray-500 ml-6 group-hover:text-gray-600">
                        {lineLengthText}
                      </p>
                    )}
                  </div>
                  <div className="flex-shrink-0 flex items-center space-x-1 opacity-60 group-hover:opacity-100">
                    <button
                      onClick={(e) => {
                        e.stopPropagation();
                        handleEditFeature(l.get("featureType"), lId);
                      }}
                      className="p-1 rounded-full text-gray-400 hover:text-blue-600 hover:bg-blue-100 transition-colors"
                      title="Редактировать детали"
                    >
                      <Pencil size={16} />
                    </button>
                    <button
                      onClick={(e) => {
                        e.stopPropagation();
                        handleDeleteFeature(lId, "route");
                      }}
                      className="p-1 rounded-full text-gray-400 hover:text-red-600 hover:bg-red-100 transition-colors"
                      title="Удалить маршрут"
                    >
                      <Trash2 size={16} />
                    </button>
                  </div>
                </div>
              );
            })
          ) : (
            <p className="text-gray-500 text-sm">Нет добавленных маршрутов.</p>
          )}
        </div>
      ),
    },
    {
      id: "sights",
      title: `Достопримечательности (${sights.length})`,
      icon: <Landmark size={20} />,
      content: (
        <div className="space-y-1 max-h-60 overflow-y-auto pr-1">
          {sights.length > 0 ? (
            sights.map((s) => {
              const sId = s.getId();
              const sName = (s.get("name") as string) || "Без названия";
              const isSelected = selectedFeature?.getId() === sId;
              return (
                <div
                  key={String(sId)}
                  className={`flex items-center justify-between p-2 rounded-md cursor-pointer group transition-colors duration-150 ${
                    isSelected
                      ? "bg-orange-100 border border-orange-300 hover:bg-orange-200"
                      : "hover:bg-blue-50"
                  }`}
                  onClick={() => handleFeatureClick(sId)}
                >
                  <div className="flex items-center text-gray-800 text-sm flex-grow mr-2 min-w-0">
                    <Landmark
                      size={16}
                      className={`mr-1.5 flex-shrink-0 ${
                        isSelected
                          ? "text-orange-500"
                          : "text-purple-500 group-hover:text-purple-600"
                      }`}
                    />
                    <span
                      className={`font-medium truncate ${
                        isSelected
                          ? "text-orange-600"
                          : "group-hover:text-purple-600"
                      }`}
                      title={sName}
                    >
                      {sName}
                    </span>
                  </div>
                  <div className="flex-shrink-0 flex items-center space-x-1 opacity-60 group-hover:opacity-100">
                    <button
                      onClick={(e) => {
                        e.stopPropagation();
                        handleEditFeature(s.get("featureType"), sId);
                      }}
                      className="p-1 rounded-full text-gray-400 hover:text-blue-600 hover:bg-blue-100 transition-colors"
                      title="Редактировать детали"
                    >
                      <Pencil size={16} />
                    </button>
                    <button
                      onClick={(e) => {
                        e.stopPropagation();
                        handleDeleteFeature(sId, "sight");
                      }}
                      className="p-1 rounded-full text-gray-400 hover:text-red-600 hover:bg-red-100 transition-colors"
                      title="Удалить достопримечательность"
                    >
                      <Trash2 size={16} />
                    </button>
                  </div>
                </div>
              );
            })
          ) : (
            <p className="text-gray-500 text-sm">
              Нет данных о достопримечательностях.
            </p>
          )}
        </div>
      ),
    },
  ];

  if (!sections.length && activeSection) {
    setActiveSection(null);
  } else if (
    sections.length > 0 &&
    !sections.find((s) => s.id === activeSection)
  ) {
    setActiveSection(sections[0]?.id || null);
  }

  return (
    <div className="w-72 relative md:w-80 bg-gray-50 shadow-lg flex flex-col border-l border-gray-200 h-[90vh]">
      <div className="p-4 bg-gray-700 text-white">
        <h2 className="text-lg font-semibold">Панель управления</h2>
      </div>
      <div className="flex-1 flex flex-col">
        <div className="flex-1 overflow-y-auto max-h-[70%]">
          {sections.map((s) => (
            <div
              key={s.id}
              className="border-b border-gray-200 last:border-b-0"
            >
              <button
                onClick={() => toggleSection(s.id)}
                className={`w-full p-3 flex items-center justify-between text-left hover:bg-gray-100 transition-colors duration-150 focus:outline-none focus:bg-gray-100 ${
                  activeSection === s.id
                    ? "bg-gray-100 text-blue-600"
                    : "text-gray-700"
                }`}
              >
                <div className="flex items-center space-x-3">
                  <span
                    className={
                      activeSection === s.id ? "text-blue-600" : "text-gray-600"
                    }
                  >
                    {s.icon}
                  </span>
                  <span className="font-medium text-sm">{s.title}</span>
                </div>
                <span
                  className={`transform transition-transform duration-200 text-gray-500 ${
                    activeSection === s.id ? "rotate-180" : ""
                  }`}
                >
                  ▼
                </span>
              </button>
              <div
                className={`overflow-hidden transition-all duration-300 ease-in-out ${
                  activeSection === s.id
                    ? "max-h-[600px] opacity-100"
                    : "max-h-0 opacity-0"
                }`}
              >
                <div className="p-3 text-sm text-gray-600 bg-white border-t border-gray-100">
                  {s.content}
                </div>
              </div>
            </div>
          ))}
        </div>
      </div>

      <button
        onClick={handleSave}
        className="m-3 w-[90%] h-[40px] flex items-center justify-center px-4 py-2 bg-blue-500 disabled:bg-blue-300 text-white rounded-md hover:bg-blue-600 transition-colors"
        disabled={isLoading}
      >
        <Save size={16} className="mr-2" />
        {isLoading ? (
          <Loader2 size={16} className="animate-spin" />
        ) : (
          "Сохранить изменения"
        )}
      </button>
    </div>
  );
};

// --- MAP PAGE COMPONENT ---
export const MapPage: React.FC = () => {
  const mapRef = useRef<HTMLDivElement | null>(null);
  const tooltipRef = useRef<HTMLDivElement | null>(null);
  const [mapServiceInstance, setMapServiceInstance] =
    useState<MapService | null>(null);
  // --- ИЗМЕНЕНИЕ: Разделение состояния загрузки ---
  const [isMapLoading, setIsMapLoading] = useState<boolean>(true); // Для рендеринга карты
  const [isDataLoading, setIsDataLoading] = useState<boolean>(true); // Для загрузки данных с API
  const [error, setError] = useState<string | null>(null);
  const [currentMapMode, setCurrentMapMode] = useState<string>("edit");
  const [mapFeatures, setMapFeatures] = useState<Feature<Geometry>[]>([]);
  const [selectedFeatureForSidebar, setSelectedFeatureForSidebar] =
    useState<Feature<Geometry> | null>(null);

  const handleFeaturesChange = useCallback(
    (feats: Feature<Geometry>[]) => setMapFeatures([...feats]),
    []
  );
  const handleFeatureSelectForSidebar = useCallback(
    (feat: Feature<Geometry> | null) => {
      setSelectedFeatureForSidebar(feat);
    },
    []
  );

  useEffect(() => {
    let service: MapService | null = null;
    if (mapRef.current && tooltipRef.current && !mapServiceInstance) {
      // Изначально оба процесса загрузки активны
      setIsMapLoading(true);
      setIsDataLoading(true);
      setError(null);

      // --- ИЗМЕНЕНИЕ: Логика загрузки данных вынесена и управляет своим состоянием ---
      const loadInitialData = async (mapService: MapService) => {
        console.log("Starting data load...");
        try {
          await Promise.all([
            mapStore.getRoutes(),
            mapStore.getStations(),
            mapStore.getSights(),
          ]);
          mapService.loadFeaturesFromApi(
            mapStore.stations,
            mapStore.routes,
            mapStore.sights
          );
        } catch (e) {
          console.error("Failed to load initial map data:", e);
          setError("Не удалось загрузить данные для карты.");
        } finally {
          // Завершаем состояние загрузки данных независимо от результата
          setIsDataLoading(false);
        }
      };

      try {
        service = new MapService(
          { ...mapConfig, target: mapRef.current },
          setIsMapLoading, // MapService теперь управляет только состоянием загрузки карты
          setError,
          setCurrentMapMode,
          handleFeaturesChange,
          handleFeatureSelectForSidebar,
          tooltipRef.current
        );
        setMapServiceInstance(service);

        // Запускаем загрузку данных
        loadInitialData(service);
      } catch (e: any) {
        console.error("MapPage useEffect error:", e);
        setError(
          `Ошибка инициализации карты: ${
            e.message || "Неизвестная ошибка"
          }. Пожалуйста, проверьте консоль.`
        );
        // В случае критической ошибки инициализации, завершаем все загрузки
        setIsMapLoading(false);
        setIsDataLoading(false);
      }
    }
    return () => {
      if (service) {
        service.destroy();
        setMapServiceInstance(null);
      }
    };
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const showLoader = isMapLoading || isDataLoading;
  const showContent = mapServiceInstance && !showLoader && !error;

  return (
    <div className="flex flex-col md:flex-row font-sans bg-gray-100 h-[90vh] overflow-hidden">
      <div className="relative flex-grow flex">
        <div
          ref={mapRef}
          id="map"
          className="flex-1 h-full relative bg-gray-200"
        >
          <div
            ref={tooltipRef}
            className="tooltip ol-tooltip bg-white text-black p-2 rounded shadow-lg text-xs whitespace-nowrap"
            style={{
              position: "absolute",
              pointerEvents: "none",
            }}
          ></div>
          {/* --- ИЗМЕНЕНИЕ: Обновленный лоадер --- */}
          {showLoader && (
            <div className="absolute inset-0 flex flex-col items-center justify-center bg-gray-500 bg-opacity-50 z-[1001]">
              <div className="w-12 h-12 border-4 border-blue-500 border-t-transparent rounded-full animate-spin mb-3"></div>
              <div className="text-md font-semibold text-white drop-shadow-md">
                {isMapLoading ? "Загрузка карты..." : "Загрузка данных..."}
              </div>
            </div>
          )}
          {error && !showLoader && (
            <div className="absolute top-1/2 left-1/2 -translate-x-1/2 -translate-y-1/2 bg-red-100 border border-red-400 text-red-700 p-6 rounded-lg shadow-lg z-[1002] text-center max-w-md">
              <h3 className="font-semibold text-lg mb-2">Произошла ошибка</h3>
              <p className="text-sm">{error}</p>
              <button
                onClick={() => window.location.reload()}
                className="mt-4 px-4 py-2 bg-red-500 text-white rounded hover:bg-red-600 transition-colors text-sm"
              >
                Перезагрузить страницу
              </button>
            </div>
          )}
        </div>
        {/* --- ИЗМЕНЕНИЕ: Условие для отображения контента --- */}
        {showContent && (
          <MapControls
            mapService={mapServiceInstance}
            activeMode={currentMapMode}
          />
        )}
      </div>
      {showContent && (
        <MapSightbar
          mapService={mapServiceInstance}
          mapFeatures={mapFeatures}
          selectedFeature={selectedFeatureForSidebar}
        />
      )}
    </div>
  );
};
