import {
  Button,
  Paper,
  TextField,
  Select,
  MenuItem,
  FormControl,
  InputLabel,
  Box,
} from "@mui/material";
import { observer } from "mobx-react-lite";
import { ArrowLeft, Save, ImagePlus } from "lucide-react";
import { Loader2 } from "lucide-react";
import { useNavigate, useParams } from "react-router-dom";
import { toast } from "react-toastify";
import { stationsStore, languageStore, cityStore } from "@shared";
import { useEffect, useState } from "react";
import { LanguageSwitcher, MediaViewer } from "@widgets";
import { SelectMediaDialog } from "@shared";

export const StationEditPage = observer(() => {
  const navigate = useNavigate();
  const [isLoading, setIsLoading] = useState(false);
  const { language } = languageStore;
  const { id } = useParams();
  const {
    editStationData,
    getEditStation,
    setEditCommonData,
    editStation,
    setLanguageEditStationData,
  } = stationsStore;
  const { cities, getCities } = cityStore;

  const handleEdit = async () => {
    try {
      setIsLoading(true);
      await editStation(Number(id));
      toast.success("Станция успешно обновлена");
    } catch (error) {
      console.error("Error updating station:", error);
      toast.error("Ошибка при обновлении станции");
    } finally {
      setIsLoading(false);
    }
  };

  useEffect(() => {
    const fetchAndSetStationData = async () => {
      if (!id) return;

      const stationId = Number(id);
      await getEditStation(stationId);
      await getCities(language);
    };

    fetchAndSetStationData();
  }, [id, language]);

  return (
    <Paper className="w-full h-full p-3 flex flex-col gap-10">
      <LanguageSwitcher />
      <div className="flex items-center gap-4">
        <button
          className="flex items-center gap-2"
          onClick={() => navigate(-1)}
        >
          <ArrowLeft size={20} />
          Назад
        </button>
      </div>

      <div className="flex flex-col gap-10 w-full items-end">
        <TextField
          fullWidth
          label="Название"
          value={editStationData[language].name || ""}
          required
          onChange={(e) =>
            setLanguageEditStationData(language, {
              name: e.target.value,
            })
          }
        />

        <FormControl fullWidth>
          <InputLabel id="direction-label">Прямой/обратный маршрут</InputLabel>
          <Select
            labelId="direction-label"
            value={editStationData.common.direction ? "Прямой" : "Обратный"}
            label="Прямой/обратный маршрут"
            onChange={(e) =>
              setEditCommonData({
                direction: e.target.value === "Прямой",
              })
            }
          >
            <MenuItem value="Прямой">Прямой</MenuItem>
            <MenuItem value="Обратный">Обратный</MenuItem>
          </Select>
        </FormControl>

        <TextField
          fullWidth
          label="Описание"
          value={editStationData[language].description || ""}
          onChange={(e) =>
            setLanguageEditStationData(language, {
              description: e.target.value,
            })
          }
        />

        <TextField
          fullWidth
          label="Адрес"
          value={editStationData[language].address || ""}
          onChange={(e) =>
            setLanguageEditStationData(language, {
              address: e.target.value,
            })
          }
        />

        <TextField
          fullWidth
          label="Координаты"
          value={`${editStationData.common.latitude} ${editStationData.common.longitude}`}
          onChange={(e) => {
            const [latitude, longitude] = e.target.value.split(" ").map(Number);
            if (!isNaN(latitude) && !isNaN(longitude)) {
              setEditCommonData({
                latitude: latitude,
                longitude: longitude,
              });
            }
          }}
        />

        <FormControl fullWidth>
          <InputLabel>Город</InputLabel>
          <Select
            value={editStationData.common.city_id || ""}
            label="Город"
            onChange={(e) => {
              const selectedCity = cities[language].find(
                (city) => city.id === e.target.value
              );
              setEditCommonData({
                city_id: e.target.value as number,
                city: selectedCity?.name || "",
              });
            }}
          >
            {cities[language].map((city) => (
              <MenuItem key={city.id} value={city.id}>
                {city.name}
              </MenuItem>
            ))}
          </Select>
        </FormControl>

        <Button
          variant="contained"
          className="w-min flex gap-2 items-center"
          startIcon={<Save size={20} />}
          onClick={handleEdit}
          disabled={isLoading || !editStationData[language]?.name}
        >
          {isLoading ? (
            <Loader2 size={20} className="animate-spin" />
          ) : (
            "Обновить"
          )}
        </Button>
      </div>
    </Paper>
  );
});
