import { DataGrid, GridColDef, GridRenderCellParams } from "@mui/x-data-grid";
import { carrierStore, languageStore, vehicleStore } from "@shared";
import { useEffect, useState } from "react";
import { observer } from "mobx-react-lite";
import { Eye, Pencil, Trash2 } from "lucide-react";
import { useNavigate } from "react-router-dom";
import { CreateButton, DeleteModal } from "@widgets";
import { VEHICLE_TYPES } from "@shared";

export const VehicleListPage = observer(() => {
  const { vehicles, getVehicles, deleteVehicle } = vehicleStore;
  const { carriers, getCarriers } = carrierStore;
  const navigate = useNavigate();
  const [isDeleteModalOpen, setIsDeleteModalOpen] = useState(false);
  const [rowId, setRowId] = useState<number | null>(null);
  const { language } = languageStore;

  useEffect(() => {
    getVehicles();
    getCarriers();
  }, [language]);

  const columns: GridColDef[] = [
    {
      field: "tail_number",
      headerName: "Бортовой номер",
      flex: 1,
    },
    {
      field: "type",
      headerName: "Тип",
      flex: 1,

      renderCell: (params: GridRenderCellParams) => {
        return (
          <div className="flex h-full gap-7  items-center">
            {VEHICLE_TYPES.find((type) => type.value === params.row.type)
              ?.label || params.row.type}
          </div>
        );
      },
    },
    {
      field: "carrier",
      headerName: "Перевозчик",
      flex: 1,
    },
    {
      field: "city",
      headerName: "Город",
      flex: 1,
    },

    {
      field: "actions",
      headerName: "Действия",
      width: 200,
      align: "center",
      headerAlign: "center",

      renderCell: (params: GridRenderCellParams) => {
        return (
          <div className="flex h-full gap-7 justify-center items-center">
            <button onClick={() => navigate(`/vehicle/${params.row.id}/edit`)}>
              <Pencil size={20} className="text-blue-500" />
            </button>
            <button onClick={() => navigate(`/vehicle/${params.row.id}`)}>
              <Eye size={20} className="text-green-500" />
            </button>
            <button
              onClick={() => {
                setIsDeleteModalOpen(true);
                setRowId(params.row.id);
              }}
            >
              <Trash2 size={20} className="text-red-500" />
            </button>
          </div>
        );
      },
    },
  ];

  const rows = vehicles.data?.map((vehicle) => ({
    id: vehicle.vehicle.id,
    tail_number: vehicle.vehicle.tail_number,
    type: vehicle.vehicle.type,
    carrier: vehicle.vehicle.carrier,
    city: carriers.data?.find(
      (carrier) => carrier.id === vehicle.vehicle.carrier_id
    )?.city,
  }));

  return (
    <>
      <div style={{ width: "100%" }}>
        <div className="flex justify-between items-center mb-10">
          <h1 className="text-2xl">Транспортные средства</h1>
          <CreateButton
            label="Создать транспортное средство"
            path="/vehicle/create"
          />
        </div>
        <DataGrid
          rows={rows}
          columns={columns}
          hideFooterPagination
          hideFooter
        />
      </div>

      <DeleteModal
        open={isDeleteModalOpen}
        onDelete={async () => {
          if (rowId) {
            await deleteVehicle(rowId);
          }
          setIsDeleteModalOpen(false);
          setRowId(null);
        }}
        onCancel={() => {
          setIsDeleteModalOpen(false);
          setRowId(null);
        }}
      />
    </>
  );
});
